#!/usr/bin/perl
use 5.016;
use strict;

use Test::More tests => 1;

use_ok('EBook::Gutenberg');

diag("Testing EBook::Gutenberg $EBook::Gutenberg::VERSION, Perl $], $^X");

# vim: expandtab shiftwidth=4
